// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, QedaElement, QedaLibrary, fs, mkdirp, path, request;

  fs = require('fs');

  mkdirp = require('mkdirp');

  path = require('path');

  request = require('sync-request');

  KicadGenerator = require('./kicad-generator');

  QedaElement = require('./qeda-element');

  QedaLibrary = (function() {
    function QedaLibrary(settings) {
      if (settings == null) {
        settings = {};
      }
      this.elementStyle = 'default';
      this.symbolStyle = 'default';
      this.patternStyle = 'default';
      this.symbol = {
        units: 'mm',
        gridSize: 2.5,
        fontSize: {
          "default": 1,
          refDes: 1,
          name: 1,
          pinNumber: 1,
          pinName: 1
        },
        lineWidth: {
          "default": 0.02
        },
        space: {
          pinName: 1
        }
      };
      this.pattern = {
        densityLevel: 'N',
        decimals: 3,
        tolerance: {
          "default": 0.1,
          fabrication: 0.05,
          placement: 0.05
        },
        clearance: {
          padToSilk: 0.2
        },
        lineWidth: {
          "default": 0.2,
          silkscreen: 0.2,
          assembly: 0.1
        },
        fontSize: {
          "default": 1,
          refDes: 1,
          value: 1
        }
      };
      this.mergeObjects(this, settings);
      this.symbolDefs = [
        {
          regexp: /DIL(\d+)/,
          handler: 'dil'
        }, {
          regexp: /QUAD/,
          handler: 'quad'
        }
      ];
      this.patternDefs = [
        {
          regexp: /QFP(\d+)P(\d+)X(\d+)X(\d+)-(\d+)/,
          handler: 'sm/qfp'
        }, {
          regexp: /SOIC(\d+)P(\d+)X(\d+)-(\d+)/,
          handler: 'sm/sop'
        }, {
          regexp: /SOP(\d+)P(\d+)X(\d+)-(\d+)/,
          handler: 'sm/sop'
        }
      ];
      this.outlineDefs = [
        {
          regexp: /JEDEC-(.*)/,
          handler: 'jedec'
        }
      ];
      this.elements = [];
    }

    QedaLibrary.prototype.add = function(element) {
      var def, newElement;
      def = this.load(element);
      if (def.abstract) {
        console.error("'" + element + "': Cannot add abstract component, use it only as base for others");
        process.exit(1);
      }
      newElement = new QedaElement(this, def);
      this.elements.push(newElement);
      return newElement;
    };

    QedaLibrary.prototype.addPatternDefinition = function(regexp, handler) {
      return this.patternDefs.push({
        regexp: regexp,
        handler: handler
      });
    };

    QedaLibrary.prototype.calculate = function() {
      var e, i, len, prop, ref;
      if (this._calculated == null) {
        this._calculated = false;
      }
      if (this._calculated) {
        return;
      }
      for (prop in this.symbol.fontSize) {
        this.symbol.fontSize[prop] *= this.symbol.gridSize;
      }
      for (prop in this.symbol.lineWidth) {
        this.symbol.lineWidth[prop] *= this.symbol.gridSize;
      }
      for (prop in this.symbol.space) {
        this.symbol.space[prop] *= this.symbol.gridSize;
      }
      ref = this.elements;
      for (i = 0, len = ref.length; i < len; i++) {
        e = ref[i];
        e.calculate(this.symbol.gridSize);
      }
      return this._calculated = true;
    };

    QedaLibrary.prototype.generateKicad = function(name) {
      var kicad;
      this.calculate();
      kicad = new KicadGenerator(this);
      return kicad.generate(name);
    };

    QedaLibrary.prototype.load = function(element) {
      var baseDef, baseElement, def, elementJson, localFile, res;
      elementJson = element.toLowerCase() + '.json';
      localFile = './library/' + elementJson;
      if (!fs.existsSync(localFile)) {
        res = request('GET', 'https://raw.githubusercontent.com/qeda/library/master/' + elementJson, {
          timeout: 3000
        });
        if (res.statusCode === 200) {
          mkdirp.sync(path.dirname(localFile));
          fs.writeFileSync(localFile, res.body);
          console.log("Loading '" + element + "': OK");
        } else {
          console.error("Loading '" + element + "': Error (" + res.statusCode + ")");
          process.exit(1);
        }
      }
      def = JSON.parse(fs.readFileSync(localFile));
      if (def.base != null) {
        baseElement = def.base;
        delete def.base;
        if (path.dirname(baseElement) === '.') {
          baseElement = path.dirname(element) + '/' + baseElement;
        }
        baseDef = this.load(baseElement);
        if (baseDef.abstract) {
          delete baseDef.abstract;
        }
        this.mergeObjects(baseDef, def);
        def = baseDef;
      }
      return def;
    };

    QedaLibrary.prototype.mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaLibrary.prototype.setElementStyle = function(style) {
      style = style.toLowerCase();
      this.elementStyle = style;
      if (this.symbolStyle === 'default' || style === 'default') {
        this.symbolStyle = style;
      }
      if (this.patternStyle === 'default' || style === 'default') {
        return this.patternStyle = style;
      }
    };

    QedaLibrary.prototype.setPatternStyle = function(style) {
      return this.patternStyle = style.toLowerCase();
    };

    QedaLibrary.prototype.setSymbolStyle = function(style) {
      return this.symbolStyle = style.toLowerCase();
    };

    return QedaLibrary;

  })();

  module.exports = QedaLibrary;

}).call(this);
